# Copyright 2025 PT Espay Debit Indonesia Koe
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# coding: utf-8

"""
    IPG Cashier Pay API

    API for enabling the user to make payment from merchant's platform with redirecting to DANA's platform.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from dana.base.model import BaseSdkModel

from pydantic import BaseModel, ConfigDict, Field, StrictStr, field_validator
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from dana.ipg.v1.models.apply_token_response_additional_info import ApplyTokenResponseAdditionalInfo
from typing import Optional, Set
from typing_extensions import Self
from pydantic import AliasGenerator
from pydantic.alias_generators import to_camel

class ApplyTokenResponse(BaseModel, BaseSdkModel):
    """
    ApplyTokenResponse
    """ # noqa: E501
    response_code: Annotated[str, Field(strict=True, max_length=7)] = Field(description="Refer to response code list:<br> * 2007400 - Successful<br> * 4007400 - Bad Request - Retry request with proper parameter<br> * 4007401 - Invalid Field Format - Retry request with proper parameter<br> * 4007402 - Invalid Mandatory Field - Retry request with proper parameter<br> * 4017400 - Unauthorized. [reason] - Retry request with proper parameter<br> * 4297400 - Too Many Requests - Retry request periodically by sending same request payload<br> * 5007400 - General Error - Retry request periodically<br> * 5007401 - Internal Server Error - Retry request periodically by sending same request payload<br> ")
    response_message: Annotated[str, Field(strict=True, max_length=150)] = Field(description="Refer to response code list")
    token_type: Optional[StrictStr] = Field(default=None, description="Token type")
    access_token: StrictStr = Field(description="Access token that can be used as user authorization")
    access_token_expiry_time: Optional[Annotated[str, Field(strict=True)]] = Field(default=None, description="Access token expiry time")
    refresh_token: Optional[StrictStr] = Field(default=None, description="Token that can be used to refresh the accessToken when it expires")
    refresh_token_expiry_time: Optional[Annotated[str, Field(strict=True)]] = Field(default=None, description="Refresh token expiry time")
    additional_info: Optional[ApplyTokenResponseAdditionalInfo] = Field(default=None)
    __properties: ClassVar[List[str]] = ["responseCode", "responseMessage", "tokenType", "accessToken", "accessTokenExpiryTime", "refreshToken", "refreshTokenExpiryTime", "additionalInfo"]

    @field_validator('access_token_expiry_time')
    def access_token_expiry_time_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\+07:00$", value):
            raise ValueError(r"must validate the regular expression /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\+07:00$/")
        return value

    @field_validator('refresh_token_expiry_time')
    def refresh_token_expiry_time_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not re.match(r"^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\+07:00$", value):
            raise ValueError(r"must validate the regular expression /^\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\+07:00$/")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=AliasGenerator(serialization_alias=to_camel, validation_alias=to_camel),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict(), separators=(',', ':'))

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ApplyTokenResponse from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of additional_info
        if self.additional_info:
            _dict['additionalInfo'] = self.additional_info.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ApplyTokenResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "responseCode": obj.get("responseCode"),
            "responseMessage": obj.get("responseMessage"),
            "tokenType": obj.get("tokenType"),
            "accessToken": obj.get("accessToken"),
            "accessTokenExpiryTime": obj.get("accessTokenExpiryTime"),
            "refreshToken": obj.get("refreshToken"),
            "refreshTokenExpiryTime": obj.get("refreshTokenExpiryTime"),
            "additionalInfo": ApplyTokenResponseAdditionalInfo.from_dict(obj["additionalInfo"]) if obj.get("additionalInfo") is not None else None
        })
        return _obj


